##*************************************************************************##
##  SCALASCA    http://www.scalasca.org/                                   ##
##*************************************************************************##
##  Copyright (c) 2023-2024                                                ##
##  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          ##
##                                                                         ##
##  This software may be modified and distributed under the terms of       ##
##  a BSD-style license.  See the COPYING file in the package base         ##
##  directory for details.                                                 ##
##*************************************************************************##


# STT_CXX_MAYBE_UNUSED_ATTR
# -------------------------
# This macro uses a best-effort approach to determine a C++ attribute
# to suppress warnings on unused variables accepted by the current C++
# compiler.  It currently checks for
#
#   [[maybe_unused]]         -  C++17 standard attribute
#   [[gnu::unused]]          -  GNU extension (also supported by others)
#   __attribute__((unused))  -  GNU extension (also supported by others)
#
# and selects the first match from this list resulting in neither a compile
# error nor a warning.  As the effect of the attribute (i.e., whether it
# really silences the warning) can only be verified if the compiler emits a
# warning by default, this may result in a false positive if no such warning
# is issued and unknown attributes are silently ignored.  However, such
# behavior has not been seen yet with any tested compiler when using standard
# compile flags (and should be harmless in any case).
#
# List of configure variables set:
#   `stt_cv_cxx_maybe_unused_attr`::
#       Accepted C++ attribute if found, 'no' otherwise
#
# List of provided config header defines:
#   `SCALASCA_MAYBE_UNUSED`::
#       Accepted C++ attribute if found, unset otherwise
#
AC_DEFUN([STT_CXX_MAYBE_UNUSED_ATTR], [
AC_LANG_PUSH([C++])
AC_CACHE_CHECK([whether $CXX supports "maybe unused" attribute],
    [stt_cv_cxx_maybe_unused_attr],
    [stt_cv_cxx_maybe_unused_attr=no
     dnl Since the C++ attribute syntax "[[...]]" interferes with M4 quoting,
     dnl we have to use quadrigraphs here...
     for _stt_tmp in "@<:@@<:@maybe_unused@:>@@:>@" \
                     "@<:@@<:@gcc::unused@:>@@:>@" \
                     "__attribute__@{:@@{:@unused@:}@@:}@"; do
         AFS_COMPILE_IFELSE_WERROR(
             [AC_LANG_PROGRAM([], [$_stt_tmp int variable = 0;])],
             [stt_cv_cxx_maybe_unused_attr=$_stt_tmp])
         AS_IF([test x$stt_cv_cxx_maybe_unused_attr != xno],
             [break])
     done])
AS_IF([test x$stt_cv_cxx_maybe_unused_attr = xno],
    [_stt_tmp=""])
AC_DEFINE_UNQUOTED([SCALASCA_MAYBE_UNUSED], [$_stt_tmp],
    [Implementation-defined attribute to suppress unused variable warnings])
AS_UNSET([_stt_tmp])
AC_LANG_POP([C++])
])
