/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Declaration of the class EventSet.
 *
 *  This header file provides the declaration of the class EventSet.
 **/
/*-------------------------------------------------------------------------*/


#ifndef PEARL_EVENTSET_H
#define PEARL_EVENTSET_H


#include <cstdint>
#include <map>
#include <vector>

#include <pearl/Event.h>


namespace pearl
{
// --- Forward declarations -------------------------------------------------

class Buffer;
class GlobalDefs;


/*-------------------------------------------------------------------------*/
/**
 *  @class   EventSet
 *  @ingroup PEARL_base
 *  @brief   Container class for a set of local events with associated roles.
 **/
/*-------------------------------------------------------------------------*/

class EventSet
{
    public:
        // --- Public methods ------------------------------

        /// @name Adding/updating events
        /// @{

        void
        addEvent(const Event&  event,
                 std::uint32_t key);

        void
        updateEvent(const Event&  event,
                    std::uint32_t key);

        /// @}
        /// @name Retrieving events
        /// @{

        Event
        getEvent(std::uint32_t key) const;

        /// @}
        /// @name Removing events
        /// @{

        void
        clear();

        /// @}
        /// @name Serializing data
        /// @{

        void
        pack(Buffer& buffer) const;

        /// @}


    private:
        // --- Type definitions ----------------------------

        /// Container type for role |-@> index mapping
        typedef std::map< std::uint32_t, std::uint32_t > KeyMap;

        /// Container type for local events
        typedef std::vector< Event > EventList;


        // --- Data members --------------------------------

        /// Mapping table key (role ID) |-@> index
        KeyMap mKeys;

        /// Current set of events
        EventList mEvents;


        // --- Private methods -----------------------------

        std::uint32_t
        storeEvent(const Event& event);
};
}    // namespace pearl


#endif    // !PEARL_EVENTSET_H
