/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2022-2023                                                **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef PEARL_MPIRMAPUTSTART_REP_H
#define PEARL_MPIRMAPUTSTART_REP_H


#include <cstdint>

#include <pearl/RmaPutStart_rep.h>


/*-------------------------------------------------------------------------*/
/**
 *  @file    MpiRmaPutStart_rep.h
 *  @ingroup PEARL_mpi
 *  @brief   Declaration of the class MpiRmaPutStart_rep.
 *
 *  This header file provides the declaration of the class MpiRmaPutStart_rep.
 **/
/*-------------------------------------------------------------------------*/


namespace pearl
{
/*-------------------------------------------------------------------------*/
/**
 *  @class   MpiRmaPutStart_rep
 *  @ingroup PEARL_mpi
 *  @brief   %Event representation for MPI_RMA_PUT_START events.
 **/
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR MpiRmaPutStart_rep
    : public RmaPutStart_rep
{
    public:
        /// @name Constructors & destructor
        /// @{

        MpiRmaPutStart_rep(timestamp_t      timestamp,
                           std::uint32_t    rma_id,
                           std::uint32_t    remote,
                           std::uint64_t    bytesSent,
                           const RmaWindow& window);

        MpiRmaPutStart_rep(const GlobalDefs& defs,
                           Buffer&           buffer);

        /// @}
        /// @name Event type information
        /// @{

        virtual event_t
        getType() const;

        virtual bool
        isOfType(event_t type) const;

        /// @}
        /// @name Access event data
        /// @{

        virtual const RmaWindow&
        getWindow() const;

        /// @}


    protected:
        /// @name Serialize event data
        /// @{

        virtual void
        pack(Buffer& buffer) const;

        /// @}
        /// @name Generate human-readable output of event data
        /// @{

        virtual std::ostream&
        output(std::ostream& stream) const;

        /// @}


    private:
        /// Memory window
        const RmaWindow& mWindow;
};
}    // namespace pearl


#endif    // !PEARL_MPIRMAPUTSTART_REP_H
