/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2022                                                     **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef PEARL_RMAPUTSTART_REP_H
#define PEARL_RMAPUTSTART_REP_H


#include <cstdint>

#include <pearl/Event_rep.h>


/*-------------------------------------------------------------------------*/
/**
 *  @file    RmaPutStart_rep.h
 *  @ingroup PEARL_base
 *  @brief   Declaration of the class RmaPutStart_rep.
 *
 *  This header file provides the declaration of the class RmaPutStart_rep.
 **/
/*-------------------------------------------------------------------------*/


namespace pearl
{
/*-------------------------------------------------------------------------*/
/**
 *  @class   RmaPutStart_rep
 *  @ingroup PEARL_base
 *  @brief   %Event representation for RMA_PUT_START events.
 **/
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR RmaPutStart_rep
    : public Event_rep
{
    public:
        /// @name Constructors & destructor
        /// @{

        RmaPutStart_rep(timestamp_t   timestamp,
                        std::uint32_t rmaId,
                        std::uint32_t remote,
                        std::uint64_t bytesSent);

        RmaPutStart_rep(const GlobalDefs& defs,
                        Buffer&           buffer);

        /// @}
        /// @name Event type information
        /// @{

        virtual event_t
        getType() const;

        virtual bool
        isOfType(event_t type) const;

        /// @}
        /// @name Access event data
        /// @{

        virtual std::uint32_t
        getRemote() const;

        virtual std::uint64_t
        getBytesSent()   const;

        virtual std::uint32_t
        getRmaId() const;

        /// @}


    protected:
        /// @name Serialize event data
        /// @{

        virtual void
        pack(Buffer& buffer) const;

        /// @}
        /// @name Generate human-readable output of event data
        /// @{

        virtual std::ostream&
        output(std::ostream& stream) const;

        /// @}


    private:
        /// Rank of target location (can be local of global, depending on the
        /// programming model)
        std::uint32_t mRemote;

        /// Length of data in bytes
        std::uint64_t mBytesSent;

        /// RMA ID
        std::uint32_t mRmaId;
};
}    // namespace pearl


#endif    // !PEARL_RMAPUTSTART_REP_H
