/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2021                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @brief Backported functionality from post-C++11 `<utility>`.
 *
 *  This header file provides a compatibility layer for selected (useful)
 *  capabilities of the C++ standard library header `<utility>` that were
 *  introduced in language standard versions newer than C++11.
 *
 *  @note
 *      This file is not intended to be feature-complete, but only contains
 *      functionality used in the code base.
 **/
/*-------------------------------------------------------------------------*/


#ifndef CXXSLB_UTILITY_H
#define CXXSLB_UTILITY_H


#include <utility>

#include <cxxslb/type_traits.h>


namespace cxxslb
{
/// @name Swap and type operations
/// @{

#if __cplusplus > 202002L
    using std::to_underlying;
#else    // C++20 and below
    /// @brief Converts an enumeration to its underlying type.
    ///
    /// Converts an enumeration value @p e to an integer value of its
    /// underlying type.
    ///
    /// @tparam T
    ///     Enumeration type
    /// @param e
    ///     Enumeration value to convert
    /// @returns
    ///     The integer value of the underlying type of @p Enum, converted
    ///     from @p e.
    ///
    /// @isocpp{2b,utility,P1682R3}
    ///
    template< class T >
    constexpr underlying_type_t< T >
    to_underlying(T e) noexcept
    {
        return static_cast< cxxslb::underlying_type_t< T > >(e);
    }
#endif    // C++20 and below

/// @}
}    // namespace cxxslb


#endif    // !CXXSLB_UTILITY_H
