/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_mpi
 *  @brief   Implementation of the class MpiGroup.
 *
 *  This file provides the implementation of the class MpiGroup.
 **/
/*-------------------------------------------------------------------------*/


#include <config.h>

#include <pearl/MpiGroup.h>

#include <pearl/MpiHelpers.h>

using namespace std;
using namespace pearl;


// --- Constructors & destructor --------------------------------------------

MpiGroup::MpiGroup(const IdType              id,
                   const String&             name,
                   const Properties&         properties,
                   const CommLocationSet&    commLocationSet,
                   const vector< uint32_t >& ranks)
    : CommSet(id, name, properties, commLocationSet, ranks),
      mHandle(MPI_GROUP_NULL)
{
}


MpiGroup::~MpiGroup()
{
    if (  (mHandle != MPI_GROUP_NULL)
       && (mHandle != MPI_GROUP_EMPTY))
    {
        MPI_Group_free(&mHandle);
    }
}


// --- Query functions ------------------------------------------------------

MPI_Group
MpiGroup::getHandle() const
{
    // Lazy generation of MPI group handle at first access
    if (mHandle == MPI_GROUP_NULL)
    {
        createMpiHandle(*this, mHandle);
    }

    return mHandle;
}
