/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2022-2023                                                **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Implementation of the class Event_rep.
 *
 *  This file provides the implementation of the class Event_rep.
 **/
/*-------------------------------------------------------------------------*/


#include <config.h>

#include <pearl/Event_rep.h>

#include <iomanip>
#include <iostream>

#include <cxxslb/utility.h>

#include <UTILS_Error.h>

#include <pearl/Buffer.h>
#include <pearl/Error.h>

#include "iomanip_detail.h"

using namespace std;
using namespace pearl;
using namespace pearl::detail;


// --- Constructors & destructor --------------------------------------------

Event_rep::~Event_rep()
{
}


// --- Access event data ----------------------------------------------------

const Region&
Event_rep::getRegion() const
{
    throw FatalError("EventRep::getRegion() -- "
                     "Oops! This method should not be called.");
}


const Callsite&
Event_rep::getCallsite() const
{
    throw FatalError("EventRep::getCallsite() -- "
                     "Oops! This method should not be called.");
}


const Communicator&
Event_rep::getComm() const
{
    throw FatalError("EventRep::getComm() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getRoot() const
{
    throw FatalError("EventRep::getRoot() -- "
                     "Oops! This method should not be called.");
}


uint64_t
Event_rep::getBytesSent() const
{
    throw FatalError("EventRep::getBytesSent() -- "
                     "Oops! This method should not be called.");
}


uint64_t
Event_rep::getBytesReceived() const
{
    throw FatalError("EventRep::getBytesReceived() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getSource() const
{
    throw FatalError("EventRep::getSource() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getDestination() const
{
    throw FatalError("EventRep::getDestination() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getTag() const
{
    throw FatalError("EventRep::getTag() -- "
                     "Oops! This method should not be called.");
}


uint64_t
Event_rep::getRequestId() const
{
    throw FatalError("EventRep::getRequestId() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getLockId() const
{
    throw FatalError("EventRep::getLockId() -- "
                     "Oops! This method should not be called.");
}


const RmaWindow&
Event_rep::getWindow() const
{
    throw FatalError("EventRep::getWindow() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getRemote() const
{
    throw FatalError("EventRep::getRemote() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getRmaId() const
{
    throw FatalError("EventRep::getRmaId() -- "
                     "Oops! This method should not be called.");
}


const MpiGroup&
Event_rep::getGroup() const
{
    throw FatalError("EventRep::getGroup() -- "
                     "Oops! This method should not be called.");
}


bool
Event_rep::isSync() const
{
    throw FatalError("EventRep::isSync() -- "
                     "Oops! This method should not be called.");
}


bool
Event_rep::isExclusive() const
{
    throw FatalError("EventRep::isExclusive() -- "
                     "Oops! This method should not be called.");
}


#ifdef PEARL_ENABLE_METRICS
    uint64_t
    Event_rep::getMetric(int index) const
    {
        throw FatalError("EventRep::getMetric(int) -- "
                         "Oops! This method should not be called.");
    }
#endif


// --- Modify event data ----------------------------------------------------

void
Event_rep::setRequestId(uint64_t)
{
    throw FatalError("EventRep::setRequestId(uint64_t) -- "
                     "Oops! This method should not be called.");
}


// --- Constructors & destructor (protected) --------------------------------

Event_rep::Event_rep(pearl::timestamp_t timestamp)
    : mTimestamp(timestamp)
{
}


Event_rep::Event_rep(const GlobalDefs& defs,
                     Buffer&           buffer)
{
    mTimestamp = buffer.getTimestamp();
}


// --- Serialize event data (protected) -------------------------------------

void
Event_rep::pack(Buffer& buffer) const
{
    buffer.putTimestamp(mTimestamp);
}


// --- Generate human-readable output of event data (protected) -------------

ostream&
Event_rep::output(ostream& stream) const
{
    // Print data
    stream << sep
           << "time: " << fixed << mTimestamp;

    return stream;
}


// --- Find previous / next request event in trace (protected) --------------

uint32_t
Event_rep::getNextReqoffs() const
{
    throw FatalError("EventRep::getNextReqoffs() -- "
                     "Oops! This method should not be called.");
}


uint32_t
Event_rep::getPrevReqoffs() const
{
    throw FatalError("EventRep::getPrevReqoffs() -- "
                     "Oops! This method should not be called.");
}


void
Event_rep::setNextReqoffs(uint32_t)
{
    throw FatalError("EventRep::setNextReqoffs(uint32_t) -- "
                     "Oops! This method should not be called.");
}


void
Event_rep::setPrevReqoffs(uint32_t)
{
    throw FatalError("EventRep::setPrevReqoffs(uint32_t) -- "
                     "Oops! This method should not be called.");
}


// --- Related functions ----------------------------------------------------

namespace pearl
{
// --- Stream I/O functions --------------------------------

ostream&
operator<<(ostream&               stream,
           const CollectiveOpType item)
{
    switch (item)
    {
        case CollectiveOpType::ALLGATHER:
            return stream << "allgather";

        case CollectiveOpType::ALLGATHERV:
            return stream << "allgatherv";

        case CollectiveOpType::ALLOCATE:
            return stream << "allocate";

        case CollectiveOpType::ALLREDUCE:
            return stream << "allreduce";

        case CollectiveOpType::ALLTOALL:
            return stream << "alltoall";

        case CollectiveOpType::ALLTOALLV:
            return stream << "alltoallv";

        case CollectiveOpType::ALLTOALLW:
            return stream << "alltoallw";

        case CollectiveOpType::BARRIER:
            return stream << "barrier";

        case CollectiveOpType::BCAST:
            return stream << "bcast";

        case CollectiveOpType::CREATE_HANDLE:
            return stream << "create_handle";

        case CollectiveOpType::CREATE_HANDLE_AND_ALLOCATE:
            return stream << "create_handle_and_allocate";

        case CollectiveOpType::DEALLOCATE:
            return stream << "deallocate";

        case CollectiveOpType::DESTROY_HANDLE:
            return stream << "destroy_handle";

        case CollectiveOpType::DESTROY_HANDLE_AND_DEALLOCATE:
            return stream << "destroy_handle_and_deallocate";

        case CollectiveOpType::EXSCAN:
            return stream << "exscan";

        case CollectiveOpType::GATHER:
            return stream << "gather";

        case CollectiveOpType::GATHERV:
            return stream << "gatherv";

        case CollectiveOpType::REDUCE:
            return stream << "reduce";

        case CollectiveOpType::REDUCE_SCATTER:
            return stream << "reduce_scatter";

        case CollectiveOpType::REDUCE_SCATTER_BLOCK:
            return stream << "reduce_scatter_block";

        case CollectiveOpType::SCAN:
            return stream << "scan";

        case CollectiveOpType::SCATTER:
            return stream << "scatter";

        case CollectiveOpType::SCATTERV:
            return stream << "scatterv";

        default:
            break;
    }

    // Given the switch statement above covers all known cases, we should
    // never get here.  But if we do, we definitely want to know...
    UTILS_BUG("Unknown collective operation type (%d)",
              static_cast< int >(cxxslb::to_underlying(item)));
}
}    // namespace pearl
