/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2024                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Implementation of the class NonBlockingCollRequest_rep.
 *
 *  This file provides the implementation of the class
 *  NonBlockingCollRequest_rep.
 **/
/*-------------------------------------------------------------------------*/


#include <config.h>

#include <pearl/NonBlockingCollRequest_rep.h>

#include <ostream>

#include "iomanip_detail.h"

using namespace std;
using namespace pearl;
using namespace pearl::detail;


// --- Constructors & destructor --------------------------------------------

NonBlockingCollRequest_rep::NonBlockingCollRequest_rep(timestamp_t timestamp,
                                                       uint64_t    requestId)
    : Event_rep(timestamp),
      mRequestId(requestId)
{
}


NonBlockingCollRequest_rep::NonBlockingCollRequest_rep(const GlobalDefs& defs,
                                                       Buffer&           buffer)
    : Event_rep(defs, buffer),
      mRequestId(PEARL_NO_REQUEST)
{
}


// --- Event type information -----------------------------------------------

event_t
NonBlockingCollRequest_rep::getType() const
{
    return NON_BLOCKING_COLLECTIVE_REQUEST;
}


bool
NonBlockingCollRequest_rep::isOfType(event_t type) const
{
    return (  (type == NON_BLOCKING_COLLECTIVE_REQUEST)
           || (type == GROUP_NONBLOCK)
           || (type == GROUP_BEGIN)
           || (type == GROUP_ALL));
}


// --- Access event data ----------------------------------------------------

uint64_t
NonBlockingCollRequest_rep::getRequestId() const
{
    return mRequestId;
}


// --- Generate human-readable output of event data (protected) -------------

ostream&
NonBlockingCollRequest_rep::output(ostream& stream) const
{
    Event_rep::output(stream);

    // Print data
    stream << sep
           << "request: " << mRequestId;

    return stream;
}


// --- Find next/prev request entries (protected) ---------------------------

uint32_t
NonBlockingCollRequest_rep::getPrevReqoffs() const
{
    return 0;
}


uint32_t
NonBlockingCollRequest_rep::getNextReqoffs() const
{
    return mNextReqoffs;
}


void
NonBlockingCollRequest_rep::setPrevReqoffs(uint32_t ptr)
{
}


void
NonBlockingCollRequest_rep::setNextReqoffs(uint32_t offs)
{
    mNextReqoffs = offs;
}
