/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOUT_LOCKEPOCHQUEUE_H
#define SCOUT_LOCKEPOCHQUEUE_H


#include <memory>
#include <queue>

#include <UTILS_Error.h>

#include <pearl/RemoteEvent.h>
#include <pearl/RemoteRmaEpoch.h>


namespace scout
{
using RemoteRmaEpochPtr = std::shared_ptr< pearl::RemoteRmaEpoch >;


class LockEpochCompare
{
    public:
        explicit
        LockEpochCompare(const bool& reverse = false)
        {
            mReverse = reverse;
        }

        bool
        operator()(const RemoteRmaEpochPtr& lhs,
                   const RemoteRmaEpochPtr& rhs) const
        {
            // Retrieve Unlock event from first epoch
            pearl::RemoteRmaEpoch::const_reverse_iterator lhsIt  = lhs->rbegin();
            pearl::RemoteRmaEpoch::const_reverse_iterator lhsEnd = lhs->rend();
            while (  (lhsIt != lhsEnd)
                  && !(*lhsIt)->isOfType(pearl::MPI_RMA_UNLOCK))
            {
                ++lhsIt;
            }
            UTILS_ASSERT(lhsIt != lhsEnd);

            // Retrieve Unlock event from second epoch
            pearl::RemoteRmaEpoch::const_reverse_iterator rhsIt  = rhs->rbegin();
            pearl::RemoteRmaEpoch::const_reverse_iterator rhsEnd = rhs->rend();
            while (  (rhsIt != rhsEnd)
                  && !(*rhsIt)->isOfType(pearl::MPI_RMA_UNLOCK))
            {
                ++rhsIt;
            }
            UTILS_ASSERT(rhsIt != rhsEnd);

            return ((*lhsIt)->getTimestamp() < (*rhsIt)->getTimestamp());
        }


    private:
        bool mReverse;
};


typedef std::priority_queue< RemoteRmaEpochPtr,
                             std::vector< RemoteRmaEpochPtr >,
                             LockEpochCompare > LockEpochQueue;
}    // namespace scout


#endif    // !SCOUT_LOCKEPOCHQUEUE_H
