/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2024                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2019-2023                                                **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOUT_PATTERNS_GEN_H
#define SCOUT_PATTERNS_GEN_H


namespace scout
{
// --- Constants ------------------------------------------------------------

const long PAT_NONE                                = -1;
const long PAT_TIME                                = 0;
const long PAT_VISITS                              = 1;
const long PAT_EXECUTION                           = 2;
const long PAT_OVERHEAD                            = 3;
const long PAT_COMP                                = 4;
const long PAT_ASYNC                               = 5;
const long PAT_MPI                                 = 6;
const long PAT_MPI_MANAGEMENT                      = 7;
const long PAT_MPI_INIT_EXIT                       = 8;
const long PAT_MPI_INIT_COMPLETION                 = 9;
const long PAT_MPI_FINALIZE_WAIT                   = 10;
const long PAT_MPI_MGMT_COMM                       = 11;
const long PAT_MPI_WAIT_COMM_CREATE                = 12;
const long PAT_MPI_COMM_CREATE_COMPLETION          = 13;
const long PAT_MPI_WAIT_COMM_FREE                  = 14;
const long PAT_MPI_MGMT_FILE                       = 15;
const long PAT_MPI_MGMT_WIN                        = 16;
const long PAT_MPI_SYNCHRONIZATION                 = 17;
const long PAT_MPI_SYNC_COLLECTIVE                 = 18;
const long PAT_MPI_BARRIER_WAIT                    = 19;
const long PAT_MPI_BARRIER_COMPLETION              = 20;
const long PAT_MPI_COMMUNICATION                   = 21;
const long PAT_MPI_POINT2POINT                     = 22;
const long PAT_MPI_LATESENDER                      = 23;
const long PAT_MPI_LATESENDER_WO                   = 24;
const long PAT_MPI_LSWO_DIFFERENT                  = 25;
const long PAT_MPI_LSWO_SAME                       = 26;
const long PAT_MPI_LATERECEIVER                    = 27;
const long PAT_MPI_COLLECTIVE                      = 28;
const long PAT_MPI_EARLYREDUCE                     = 29;
const long PAT_MPI_EARLYSCAN                       = 30;
const long PAT_MPI_LATEBROADCAST                   = 31;
const long PAT_MPI_WAIT_NXN                        = 32;
const long PAT_MPI_NXN_COMPLETION                  = 33;
const long PAT_MPI_IO                              = 34;
const long PAT_MPI_IO_INDIVIDUAL                   = 35;
const long PAT_MPI_IO_COLLECTIVE                   = 36;
const long PAT_OMP_IDLE_THREADS                    = 37;
const long PAT_OMP_LIMITED_PARALLELISM             = 38;
const long PAT_OMP_TIME                            = 39;
const long PAT_OMP_MANAGEMENT                      = 40;
const long PAT_OMP_FORK                            = 41;
const long PAT_OMP_SYNCHRONIZATION                 = 42;
const long PAT_OMP_BARRIER                         = 43;
const long PAT_OMP_EBARRIER                        = 44;
const long PAT_OMP_EBARRIER_WAIT                   = 45;
const long PAT_OMP_IBARRIER                        = 46;
const long PAT_OMP_IBARRIER_WAIT                   = 47;
const long PAT_OMP_CRITICAL                        = 48;
const long PAT_OMP_LOCK_CONTENTION_CRITICAL        = 49;
const long PAT_OMP_LOCK_API                        = 50;
const long PAT_OMP_LOCK_CONTENTION_API             = 51;
const long PAT_OMP_ORDERED                         = 52;
const long PAT_OMP_TASKWAIT                        = 53;
const long PAT_OMP_FLUSH                           = 54;
const long PAT_PTHREAD_TIME                        = 55;
const long PAT_PTHREAD_MANAGEMENT                  = 56;
const long PAT_PTHREAD_SYNCHRONIZATION             = 57;
const long PAT_PTHREAD_LOCK_API                    = 58;
const long PAT_PTHREAD_LOCK_CONTENTION_MUTEX_LOCK  = 59;
const long PAT_PTHREAD_CONDITIONAL                 = 60;
const long PAT_PTHREAD_LOCK_CONTENTION_CONDITIONAL = 61;
const long PAT_THREAD_LOCK_CONTENTION              = 62;
const long PAT_OPENACC_TIME                        = 63;
const long PAT_OPENACC_SETUP                       = 64;
const long PAT_OPENACC_COMM                        = 65;
const long PAT_OPENACC_SYNC                        = 66;
const long PAT_OPENACC_KERNEL_LAUNCHES             = 67;
const long PAT_OPENCL_KERNEL_EXECUTIONS            = 68;
const long PAT_OPENCL_TIME                         = 69;
const long PAT_OPENCL_SETUP                        = 70;
const long PAT_OPENCL_COMM                         = 71;
const long PAT_OPENCL_SYNC                         = 72;
const long PAT_OPENCL_KERNEL_LAUNCHES              = 73;
const long PAT_CUDA_KERNEL_EXECUTIONS              = 74;
const long PAT_CUDA_TIME                           = 75;
const long PAT_CUDA_SETUP                          = 76;
const long PAT_CUDA_COMM                           = 77;
const long PAT_CUDA_SYNC                           = 78;
const long PAT_CUDA_KERNEL_LAUNCHES                = 79;
const long PAT_SYNCS                               = 80;
const long PAT_SYNCS_P2P                           = 81;
const long PAT_SYNCS_SEND                          = 82;
const long PAT_SYNCS_RECV                          = 83;
const long PAT_SYNCS_COLL                          = 84;
const long PAT_SYNCS_RMA                           = 85;
const long PAT_SYNCS_RMA_ACTIVE                    = 86;
const long PAT_SYNCS_RMA_PASSIVE                   = 87;
const long PAT_COMMS                               = 88;
const long PAT_COMMS_P2P                           = 89;
const long PAT_COMMS_SEND                          = 90;
const long PAT_COMMS_RECV                          = 91;
const long PAT_COMMS_COLL                          = 92;
const long PAT_COMMS_CXCH                          = 93;
const long PAT_COMMS_CSRC                          = 94;
const long PAT_COMMS_CDST                          = 95;
const long PAT_COMMS_RMA                           = 96;
const long PAT_COMMS_RMA_PUTS                      = 97;
const long PAT_COMMS_RMA_GETS                      = 98;
const long PAT_COMMS_RMA_ATOMICS                   = 99;
const long PAT_BYTES                               = 100;
const long PAT_BYTES_P2P                           = 101;
const long PAT_BYTES_SENT                          = 102;
const long PAT_BYTES_RCVD                          = 103;
const long PAT_BYTES_COLL                          = 104;
const long PAT_BYTES_COUT                          = 105;
const long PAT_BYTES_CIN                           = 106;
const long PAT_BYTES_RMA                           = 107;
const long PAT_BYTES_PUT                           = 108;
const long PAT_BYTES_GET                           = 109;
const long PAT_MPI_SLR_COUNT                       = 110;
const long PAT_MPI_SLS_COUNT                       = 111;
const long PAT_MPI_SLSWO_COUNT                     = 112;
const long PAT_MPI_CLR_COUNT                       = 113;
const long PAT_MPI_CLS_COUNT                       = 114;
const long PAT_MPI_CLSWO_COUNT                     = 115;
const long PAT_MPI_FILE_OPS                        = 116;
const long PAT_MPI_FILE_IOPS                       = 117;
const long PAT_MPI_FILE_IROPS                      = 118;
const long PAT_MPI_FILE_IWOPS                      = 119;
const long PAT_MPI_FILE_COPS                       = 120;
const long PAT_MPI_FILE_CROPS                      = 121;
const long PAT_MPI_FILE_CWOPS                      = 122;
const long PAT_MPI_RMA_WAIT_AT_CREATE              = 123;
const long PAT_MPI_RMA_WAIT_AT_FREE                = 124;
const long PAT_MPI_RMA_SYNCHRONIZATION             = 125;
const long PAT_MPI_RMA_SYNC_ACTIVE                 = 126;
const long PAT_MPI_RMA_SYNC_LATE_POST              = 127;
const long PAT_MPI_RMA_EARLY_WAIT                  = 128;
const long PAT_MPI_RMA_LATE_COMPLETE               = 129;
const long PAT_MPI_RMA_WAIT_AT_FENCE               = 130;
const long PAT_MPI_RMA_EARLY_FENCE                 = 131;
const long PAT_MPI_RMA_SYNC_PASSIVE                = 132;
const long PAT_MPI_RMA_SYNC_LOCK_COMPETITION       = 133;
const long PAT_MPI_RMA_SYNC_WAIT_FOR_PROGRESS      = 134;
const long PAT_MPI_RMA_COMMUNICATION               = 135;
const long PAT_MPI_RMA_COMM_LATE_POST              = 136;
const long PAT_MPI_RMA_COMM_LOCK_COMPETITION       = 137;
const long PAT_MPI_RMA_COMM_WAIT_FOR_PROGRESS      = 138;
const long PAT_MPI_RMA_PAIRSYNC_COUNT              = 139;
const long PAT_MPI_RMA_PAIRSYNC_UNNEEDED_COUNT     = 140;
const long PAT_IMBALANCE                           = 141;
const long PAT_IMBALANCE_ABOVE                     = 142;
const long PAT_IMBALANCE_ABOVE_SINGLE              = 143;
const long PAT_IMBALANCE_BELOW                     = 144;
const long PAT_IMBALANCE_BELOW_BYPASS              = 145;
const long PAT_IMBALANCE_BELOW_SINGULARITY         = 146;
const long PAT_STATISTICS                          = 147;
const long PAT_CRITICAL_PATH                       = 148;
const long PAT_CRITICAL_PATH_IMBALANCE             = 149;
const long PAT_PERFORMANCE_IMPACT                  = 150;
const long PAT_PERFORMANCE_IMPACT_CRITICALPATH     = 151;
const long PAT_CRITICAL_PATH_ACTIVITIES            = 152;
const long PAT_CRITICAL_IMBALANCE_IMPACT           = 153;
const long PAT_INTRA_PARTITION_IMBALANCE           = 154;
const long PAT_INTER_PARTITION_IMBALANCE           = 155;
const long PAT_NON_CRITICAL_PATH_ACTIVITIES        = 156;
const long PAT_DELAY                               = 157;
const long PAT_DELAY_MPI                           = 158;
const long PAT_DELAY_MANAGEMENT                    = 159;
const long PAT_DELAY_EXIT                          = 160;
const long PAT_DELAY_FINALIZE_AGGREGATE            = 161;
const long PAT_DELAY_FINALIZE                      = 162;
const long PAT_DELAY_FINALIZE_LONGTERM             = 163;
const long PAT_DELAY_MGMT_COMM                     = 164;
const long PAT_DELAY_COMM_CREATE_AGGREGATE         = 165;
const long PAT_DELAY_COMM_CREATE                   = 166;
const long PAT_DELAY_COMM_CREATE_LONGTERM          = 167;
const long PAT_DELAY_COMM_FREE_AGGREGATE           = 168;
const long PAT_DELAY_COMM_FREE                     = 169;
const long PAT_DELAY_COMM_FREE_LONGTERM            = 170;
const long PAT_DELAY_P2P                           = 171;
const long PAT_DELAY_LATESENDER_AGGREGATE          = 172;
const long PAT_DELAY_LATESENDER                    = 173;
const long PAT_DELAY_LATESENDER_LONGTERM           = 174;
const long PAT_DELAY_LATERECEIVER_AGGREGATE        = 175;
const long PAT_DELAY_LATERECEIVER                  = 176;
const long PAT_DELAY_LATERECEIVER_LONGTERM         = 177;
const long PAT_DELAY_COLLECTIVE                    = 178;
const long PAT_DELAY_BARRIER_AGGREGATE             = 179;
const long PAT_DELAY_BARRIER                       = 180;
const long PAT_DELAY_BARRIER_LONGTERM              = 181;
const long PAT_DELAY_N2N_AGGREGATE                 = 182;
const long PAT_DELAY_N2N                           = 183;
const long PAT_DELAY_N2N_LONGTERM                  = 184;
const long PAT_DELAY_12N_AGGREGATE                 = 185;
const long PAT_DELAY_12N                           = 186;
const long PAT_DELAY_12N_LONGTERM                  = 187;
const long PAT_DELAY_N21_AGGREGATE                 = 188;
const long PAT_DELAY_N21                           = 189;
const long PAT_DELAY_N21_LONGTERM                  = 190;
const long PAT_DELAY_OMP                           = 191;
const long PAT_DELAY_OMPBARRIER_AGGREGATE          = 192;
const long PAT_DELAY_OMPBARRIER                    = 193;
const long PAT_DELAY_OMPBARRIER_LONGTERM           = 194;
const long PAT_DELAY_OMPIDLE_AGGREGATE             = 195;
const long PAT_DELAY_OMPIDLE                       = 196;
const long PAT_DELAY_OMPIDLE_LONGTERM              = 197;
const long PAT_WAITSTATES_DIRECT_VS_INDIRECT       = 198;
const long PAT_MPI_WAIT_DIRECT                     = 199;
const long PAT_MPI_WAIT_DIRECT_LATESENDER          = 200;
const long PAT_MPI_WAIT_DIRECT_LATERECEIVER        = 201;
const long PAT_MPI_WAIT_INDIRECT                   = 202;
const long PAT_MPI_WAIT_INDIRECT_LATESENDER        = 203;
const long PAT_MPI_WAIT_INDIRECT_LATERECEIVER      = 204;
const long PAT_WAITSTATES_PROPAGATING_VS_TERMINAL  = 205;
const long PAT_MPI_WAIT_PROPAGATING                = 206;
const long PAT_MPI_WAIT_PROPAGATING_LS             = 207;
const long PAT_MPI_WAIT_PROPAGATING_LR             = 208;
const long PAT_MPI_WAIT_TERMINAL                   = 209;
const long PAT_MPI_WAIT_TERMINAL_LS                = 210;
const long PAT_MPI_WAIT_TERMINAL_LR                = 211;


// --- Forward declarations -------------------------------------------------

class AnalyzeTask;


// --- Pattern registration -------------------------------------------------

void
create_patterns(AnalyzeTask* analyzer);
}    // namespace scout


#endif    // !SCOUT_PATTERNS_GEN_H
