/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2024                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOUT_ROLES_H
#define SCOUT_ROLES_H


#include <cstdint>


/*-------------------------------------------------------------------------*/
/**
 *  @file  Roles.h
 *  @brief Declaration of constants for event roles.
 *
 *  This header file provides a set of numerical constants which provide
 *  symbolic names for the different roles that can be attributed to events.
 *  These are used when transferring events between processes using the
 *  pearl::EventSet and pearl::RemoteEventSet classes, respectively.
 **/
/*-------------------------------------------------------------------------*/


namespace scout
{
// --- Role ID constants ----------------------------------------------------

const std::uint32_t ROLE_SEND       = 0;
const std::uint32_t ROLE_RECV       = 1;
const std::uint32_t ROLE_ENTER_SEND = 2;
const std::uint32_t ROLE_ENTER_RECV = 3;
const std::uint32_t ROLE_LEAVE_SEND = 4;
const std::uint32_t ROLE_LEAVE_RECV = 5;

const std::uint32_t ROLE_SEND_LS = 6;
const std::uint32_t ROLE_RECV_LS = 7;

const std::uint32_t ROLE_SEND_LSWO = 12;
const std::uint32_t ROLE_RECV_LSWO = 13;

const std::uint32_t ROLE_ENTER_SENDCMP_LR = 14;
const std::uint32_t ROLE_RECV_LR          = 15;
const std::uint32_t ROLE_ENTER_SENDCMP    = 16;
const std::uint32_t ROLE_ENTER_RECVREQ    = 17;
const std::uint32_t ROLE_LEAVE_SENDCMP    = 18;
const std::uint32_t ROLE_LEAVE_RECVREQ    = 19;

const std::uint32_t ROLE_ENTER_COLL = 20;
const std::uint32_t ROLE_BEGIN_COLL = 21;
const std::uint32_t ROLE_END_COLL   = 22;
const std::uint32_t ROLE_LEAVE_COLL = 23;

const std::uint32_t ROLE_ENTER_RMA_START    = 24;
const std::uint32_t ROLE_LEAVE_RMA_START    = 25;
const std::uint32_t ROLE_LEAVE_RMA_COMPLETE = 26;

const std::uint32_t ROLE_LEAVE_SEND_SYNC    = 30;
const std::uint32_t ROLE_LEAVE_RECV_SYNC    = 31;
const std::uint32_t ROLE_LEAVE_RECVREQ_SYNC = 32;

const std::uint32_t BUFFER_SYNC         = 101;
const std::uint32_t BUFFER_CRITICALPATH = 111;

// Keys under which remote delay costs are stored in MPI point-to-point
// transfer buffers.  Name suffixes have to stay in sync with the
// corresponding (PRE_)DELAY user events!
const std::uint32_t BUFFER_DELAY                      = 121;
const std::uint32_t BUFFER_RCOST_LATE_SENDER          = 122;
const std::uint32_t BUFFER_RCOST_LATE_RECEIVER        = 123;
const std::uint32_t BUFFER_RCOST_MPI_BARRIER          = 124;
const std::uint32_t BUFFER_RCOST_MPI_COLL_12N         = 125;
const std::uint32_t BUFFER_RCOST_MPI_COLL_N21         = 126;
const std::uint32_t BUFFER_RCOST_MPI_COLL_N2N         = 127;
const std::uint32_t BUFFER_RCOST_MPI_COLL_COMM_CREATE = 128;
const std::uint32_t BUFFER_RCOST_MPI_COLL_COMM_FREE   = 129;
const std::uint32_t BUFFER_RCOST_MPI_FINALIZE         = 130;
const std::uint32_t BUFFER_RCOST_OMP_BARRIER          = 131;
const std::uint32_t BUFFER_RCOST_OMP_IDLE             = 132;

const std::uint32_t BUFFER_PROPWAIT = 141;

// Keys under which input delay and scale values are stored.  Name suffixes
// have to stay in sync with the corresponding (PRE_)DELAY user events!
const std::uint32_t KEY_WAITTIME                   = 201;
const std::uint32_t KEY_RCOST_LATE_SENDER          = 202;
const std::uint32_t KEY_RCOST_LATE_RECEIVER        = 203;
const std::uint32_t KEY_RCOST_MPI_BARRIER          = 204;
const std::uint32_t KEY_RCOST_MPI_COLL_12N         = 205;
const std::uint32_t KEY_RCOST_MPI_COLL_N21         = 206;
const std::uint32_t KEY_RCOST_MPI_COLL_N2N         = 207;
const std::uint32_t KEY_RCOST_MPI_COLL_COMM_CREATE = 208;
const std::uint32_t KEY_RCOST_MPI_COLL_COMM_FREE   = 209;
const std::uint32_t KEY_RCOST_MPI_FINALIZE         = 210;
const std::uint32_t KEY_RCOST_OMP_BARRIER          = 211;
const std::uint32_t KEY_RCOST_OMP_IDLE             = 212;
}    // namespace scout


#endif    // !SCOUT_ROLES_H
