/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @brief Declaration of global settings variables.
 *
 *  This header file provides the declaration of global variables providing
 *  configuration settings of the Scalasca Trace Analyzer.
 **/
/*-------------------------------------------------------------------------*/


#ifndef SCOUT_SETTINGS_H
#define SCOUT_SETTINGS_H


#include <string>


namespace scout
{
// --- Global variables -----------------------------------------------------

/// Experiment archive directory
extern std::string archiveDirectory;

/// Flag indicating whether metric statistics and most-severe instance
/// tracking should be enabled
extern bool enableStatistics;

/// Flag indicating whether critical-path analysis should be enabled
extern bool enableCriticalPath;

/// Flag indicating whether delay analysis should be enabled
extern bool enableDelayAnalysis;

/// Flag indicating whether asynchronous message handling (a.k.a. active
/// messages) should be enabled
extern bool enableAsynchronous;

/// Flag indicating whether only a single-pass replay should be used
extern bool enableSinglePass;
}    // namespace scout


#endif    // !SCOUT_SETTINGS_H
