/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2022                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "WaitstateCandidates.h"

#include <pearl/CallbackManager.h>
#include <pearl/LocalData.h>
#include <pearl/RemoteData.h>

#include "Roles.h"
#include "user_events.h"

using namespace pearl;
using namespace scout;


// --------------------------------------------------------------------------
//
//  WaitstateCandidate
//
// --------------------------------------------------------------------------

// --- Notification functions -----------------------------------------------

void
WaitstateCandidate::notifyCandidate(const CallbackManager& cbmanager,
                                    const Event&           event,
                                    CbData* const          data) const
{
}


// --------------------------------------------------------------------------
//
//  LateSenderCandidate
//
// --------------------------------------------------------------------------

// --- Notification functions -----------------------------------------------

void
LateSenderCandidate::notifyCandidate(const CallbackManager& cbmanager,
                                     const Event&           event,
                                     CbData* const          data) const
{
    cbmanager.notify(LATE_SENDER_CANDIDATE, event, data);
}


void
LateSenderCandidate::notifyInstance(const CallbackManager& cbmanager,
                                    const Event&           event,
                                    CbData* const          data) const
{
    data->mRemote->addEvent(mSend, ROLE_SEND_LS);
    data->mLocal->addEvent(mRecv, ROLE_RECV_LS);

    cbmanager.notify(LATE_SENDER, event, data);
}


// --------------------------------------------------------------------------
//
//  LswoCandidate
//
// --------------------------------------------------------------------------

// --- Notification functions -----------------------------------------------

void
LswoCandidate::notifyCandidate(const CallbackManager& cbmanager,
                               const Event&           event,
                               CbData* const          data) const
{
}


void
LswoCandidate::notifyInstance(const CallbackManager& cbmanager,
                              const Event&           event,
                              CbData* const          data) const
{
    data->mRemote->updateEvent(mSend, ROLE_SEND_LSWO);
    data->mLocal->updateEvent(mRecv, ROLE_RECV_LSWO);

    cbmanager.notify(LATE_SENDER_WO, event, data);
}
